@@ -1,10 +1,16 @@
 import numpy as np
 import mxnet as mx
-import os, argparse, glob
+import os, argparse, glob, pathlib
 import skimage
 from natsort import natsorted
 
-from cellpose import utils, models, gui
+from . import utils, models, io
+
+try:
+    from cellpose import gui 
+    GUI_ENABLED = True 
+except:
+    GUI_ENABLED = False
 
 def get_image_files(folder):
     image_names = []
@@ -17,6 +23,7 @@ def get_image_files(folder):
     return image_names
         
 def get_label_files(image_names, imf, mask_filter):
+    nimg = len(image_names)
     label_names = [os.path.splitext(image_names[n])[0] for n in range(nimg)]
     if len(imf) > 0:
         label_names = [label_names[n][:-len(imf)] for n in range(nimg)]
@@ -41,6 +48,8 @@ if __name__ == '__main__':
     # settings for running cellpose
     parser.add_argument('--pretrained_model', required=False, 
                         default='cyto', type=str, help='model to use')
+    parser.add_argument('--unet', required=False, 
+                        default=0, type=int, help='run standard unet instead of cellpose flow output')
     parser.add_argument('--chan', required=False, 
                         default=0, type=int, help='channel to segment; 0: GRAY, 1: RED, 2: GREEN, 3: BLUE')
     parser.add_argument('--chan2', required=False, 
@@ -55,6 +64,8 @@ if __name__ == '__main__':
                         default='_masks', type=str, help='end string for masks to run on')
     parser.add_argument('--test_dir', required=False, 
                         default=[], type=str, help='folder containing test data (optional)')
+    parser.add_argument('--learning_rate', required=False, 
+                        default=0.2, type=float, help='learning rate')
     parser.add_argument('--n_epochs', required=False, 
                         default=500, type=int, help='number of epochs')
     parser.add_argument('--batch_size', required=False, 
@@ -63,10 +74,9 @@ if __name__ == '__main__':
     args = parser.parse_args()
 
     if len(args.dir)==0:
-        try:
-            from cellpose import gui
-        except:
-            print('pyqt is not installed')
+        if not GUI_ENABLED:
+            print('ERROR: GUI version is not installed, to install, run')
+            print('     pip install cellpose[gui]')
         gui.run()
 
     else:
@@ -91,7 +101,6 @@ if __name__ == '__main__':
                 imn.append(im)
         image_names = imn
         nimg = len(image_names)
-        #nimg = 
         images = [skimage.io.imread(image_names[n]) for n in range(nimg)]
 
         if args.use_gpu:
@@ -101,85 +110,92 @@ if __name__ == '__main__':
         else:
             device = mx.cpu()
         print('>>>> using %s'%(['CPU', 'GPU'][use_gpu]))
-
-        if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
-            cp_path = os.path.dirname(os.path.realpath(__file__))
-            if not args.train:
-                model_list = ['%s_%d'%(args.pretrained_model, i) for i in range(4)]
-                cpmodel_path = [os.path.abspath(os.path.join(cp_path, 'models/', model_list[i]))
-                                    for i in range(len(model_list))]
-            else:
-                model_list = ['%s_0'%(args.pretrained_model)]
-                cpmodel_path = os.path.join(cp_path, 'models/', model_list[0])
-            
-            szmodel_path = os.path.abspath(os.path.join(cp_path, 'models/', 'size_%s_0.npy'%args.pretrained_model))
-            if args.pretrained_model=='cyto':
-                szmean = 27.
-            else:
-                szmean = 15.
-        else:
-            cpmodel_path = args.pretrained_model
-            szmodel_path = None
-            szmean = 27.
-            if args.all_channels:
-                channels = None                
+        model_dir = pathlib.Path.home().joinpath('.cellpose', 'models')              
 
         if not args.train:
-            model = models.Cellpose(device=device, 
-                                    pretrained_model=cpmodel_path,
-                                    pretrained_size=szmodel_path,
-                                    diam_mean=szmean
-                                   )
-            if args.diameter==0:
-                diameter = None
-                if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
+            if not args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
+                cpmodel_path = args.pretrained_model
+                if not os.path.exists(cpmodel_path):
+                    print('model path does not exist, using cyto model')
+                    args.pretrained_model = 'cyto'
+
+            if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
+                model = models.Cellpose(device=device, model_type=args.pretrained_model)
+                    
+                if args.diameter==0:
+                    diameter = None
                     print('>>>> estimating diameter for each image')
                 else:
-                    print('>>>> using user-specified model, no auto-diameter estimation available')
+                    diameter = args.diameter
+                    print('>>>> using diameter %0.2f for all images'%diameter)
+
+                cstr0 = ['GRAY', 'RED', 'GREEN', 'BLUE']
+                cstr1 = ['NONE', 'RED', 'GREEN', 'BLUE']
+                print('running cellpose on %d images using chan_to_seg %s and chan (opt) %s'%
+                        (nimg, cstr0[channels[0]], cstr1[channels[1]]))
+                
+                masks, flows, _, diams = model.eval(images, channels=channels, diameter=diameter)
+                
             else:
-                diameter = args.diameter
-                print('using diameter %0.2f for all images'%diameter)
-            cstr0 = ['GRAY', 'RED', 'GREEN', 'BLUE']
-            cstr1 = ['NONE', 'RED', 'GREEN', 'BLUE']
-            print('running cellpose on %d images using chan_to_seg %s and chan (opt) %s'%
-                    (nimg, cstr0[channels[0]], cstr1[channels[1]]))
-            
-            masks, flows, _, diams = model.eval(images, channels=channels, diameter=diameter)
+                if args.all_channels:
+                    channels = None  
+                if args.diameter==0:
+                    print('>>>> using user-specified model, no auto-diameter estimation available')
+                model = models.CellposeModel(device=device, pretrained_model=cpmodel_path)
+                masks, flows, _ = model.eval(images, channels=channels)
+                diams = 30. * np.ones(len(images)) 
+                  
             print('>>>> saving results')
-            utils.masks_flows_to_seg(images, masks, flows, diams, channels, image_names)
+            io.masks_flows_to_seg(images, masks, flows, diams, image_names, channels)
             if args.save_png:
-                utils.save_to_png(images, masks, flows, image_names)
+                io.save_to_png(images, masks, flows, image_names)
                     
         else:
+            if args.pretrained_model=='cyto' or args.pretrained_model=='nuclei':
+                cpmodel_path = os.fspath(model_dir.joinpath('%s_0'%(args.pretrained_model)))
+                if args.pretrained_model=='cyto':
+                    szmean = 27.
+                else:
+                    szmean = 15.
+            else:
+                szmean = 27.
+            
+            if args.all_channels:
+                channels = None  
+
             label_names = get_label_files(image_names, imf, args.mask_filter)
+            nimg = len(image_names)
             labels = [skimage.io.imread(label_names[n]) for n in range(nimg)]
             if not os.path.exists(cpmodel_path):
-                cpmodel_path = None
+                cpmodel_path = False
                 print('>>>> training from scratch')
+                if args.diameter==0:
+                    rescale = False 
+                    print('>>>> median diameter set to 0 => no rescaling during training')
+                else:
+                    rescale = True
+                    szmean = args.diameter * (np.pi**0.5/2)
             else:
+                rescale = True
+                args.diameter = szmean / (np.pi**0.5/2)
                 print('>>>> training starting with pretrained_model %s'%cpmodel_path)
+            if rescale:
+                print('>>>> rescaling diameter for each training image to %0.1f'%args.diameter)
+                
+            
 
             test_images, test_labels = None, None
             if len(args.test_dir) > 0:
                 image_names_test = get_image_files(args.test_dir)
                 label_names_test = get_label_files(image_names_test, imf, args.mask_filter)
+                nimg = len(image_names_test)
                 test_images = [skimage.io.imread(image_names_test[n]) for n in range(nimg)]
                 test_labels = [skimage.io.imread(label_names_test[n]) for n in range(nimg)]
-                
-            model = models.CellposeModel(device=device, pretrained_model=cpmodel_path)
-            model.train(images, labels, test_images, test_labels, 
-                        channels=channels, save_path=os.path.realpath(args.dir))
-
+            print('>>>> %s model'%(['cellpose', 'unet'][args.unet]))    
+            model = models.CellposeModel(device=device, 
+                                         unet=args.unet, 
+                                         pretrained_model=cpmodel_path, 
+                                         diam_mean=szmean)
             
-
-    
-
-#Or to train on cytoplasmic images starting with a pretrained model from cellpose (cyto or nuclei):
-#~~~
-#python -m cellpose --train --dir ~/images_cyto/train/ --pretrained_cyto cyto
-#~~~
-
-#You can specify the full path to a pretrained model to use:
-#~~~
-#python -m cellpose --dir ~/images_cyto/test/ --pretrained_model ~/images_cyto/test/model/cellpose_35_0 --save_png
-#~~~
+            model.train(images, labels, test_images, test_labels, learning_rate=args.learning_rate,
+                        channels=channels, save_path=os.path.realpath(args.dir), rescale=rescale)
\ No newline at end of file
