@@ -131,7 +131,7 @@ class RGBRadioButtons(QtGui.QButtonGroup):
         super(RGBRadioButtons, self).__init__()
         parent.color = 0
         self.parent = parent
-        self.bstr = ["image", "flowsXY", "cellprob"]#, "flowsZ"]
+        self.bstr = ["image", "flowsXY", "cellprob", "flowsZ"]
         #self.buttons = QtGui.QButtonGroup()
         self.dropdown = []
         for b in range(len(self.bstr)):
@@ -154,48 +154,50 @@ class RGBRadioButtons(QtGui.QButtonGroup):
 
 class ViewBoxNoRightDrag(pg.ViewBox):
     def __init__(self, parent=None, border=None, lockAspect=False, enableMouse=True, invertY=False, enableMenu=True, name=None, invertX=False):
-        pg.ViewBox.__init__(self, parent, border, lockAspect, enableMouse,
+        pg.ViewBox.__init__(self, None, border, lockAspect, enableMouse,
                             invertY, enableMenu, name, invertX)
+        self.parent = parent
 
     def mouseDragEvent(self, ev, axis=None):
         ## if axis is specified, event will only affect that axis.
-        ev.accept()  ## we accept all buttons
+        if self.parent is None or (self.parent is not None and not self.parent.in_stroke):
+            ev.accept()  ## we accept all buttons
 
-        pos = ev.pos()
-        lastPos = ev.lastPos()
-        dif = pos - lastPos
-        dif = dif * -1
+            pos = ev.pos()
+            lastPos = ev.lastPos()
+            dif = pos - lastPos
+            dif = dif * -1
 
-        ## Ignore axes if mouse is disabled
-        mouseEnabled = np.array(self.state['mouseEnabled'], dtype=np.float)
-        mask = mouseEnabled.copy()
-        if axis is not None:
-            mask[1-axis] = 0.0
+            ## Ignore axes if mouse is disabled
+            mouseEnabled = np.array(self.state['mouseEnabled'], dtype=np.float)
+            mask = mouseEnabled.copy()
+            if axis is not None:
+                mask[1-axis] = 0.0
 
-        ## Scale or translate based on mouse button
-        if ev.button() & (QtCore.Qt.LeftButton | QtCore.Qt.MidButton):
-            if self.state['mouseMode'] == pg.ViewBox.RectMode:
-                if ev.isFinish():  ## This is the final move in the drag; change the view scale now
-                    #print "finish"
-                    self.rbScaleBox.hide()
-                    ax = QtCore.QRectF(Point(ev.buttonDownPos(ev.button())), Point(pos))
-                    ax = self.childGroup.mapRectFromParent(ax)
-                    self.showAxRect(ax)
-                    self.axHistoryPointer += 1
-                    self.axHistory = self.axHistory[:self.axHistoryPointer] + [ax]
+            ## Scale or translate based on mouse button
+            if ev.button() & (QtCore.Qt.LeftButton | QtCore.Qt.MidButton):
+                if self.state['mouseMode'] == pg.ViewBox.RectMode:
+                    if ev.isFinish():  ## This is the final move in the drag; change the view scale now
+                        #print "finish"
+                        self.rbScaleBox.hide()
+                        ax = QtCore.QRectF(Point(ev.buttonDownPos(ev.button())), Point(pos))
+                        ax = self.childGroup.mapRectFromParent(ax)
+                        self.showAxRect(ax)
+                        self.axHistoryPointer += 1
+                        self.axHistory = self.axHistory[:self.axHistoryPointer] + [ax]
+                    else:
+                        ## update shape of scale box
+                        self.updateScaleBox(ev.buttonDownPos(), ev.pos())
                 else:
-                    ## update shape of scale box
-                    self.updateScaleBox(ev.buttonDownPos(), ev.pos())
-            else:
-                tr = dif*mask
-                tr = self.mapToView(tr) - self.mapToView(Point(0,0))
-                x = tr.x() if mask[0] == 1 else None
-                y = tr.y() if mask[1] == 1 else None
+                    tr = dif*mask
+                    tr = self.mapToView(tr) - self.mapToView(Point(0,0))
+                    x = tr.x() if mask[0] == 1 else None
+                    y = tr.y() if mask[1] == 1 else None
 
-                self._resetTarget()
-                if x is not None or y is not None:
-                    self.translateBy(x=x, y=y)
-                self.sigRangeChangedManually.emit(self.state['mouseEnabled'])
+                    self._resetTarget()
+                    if x is not None or y is not None:
+                        self.translateBy(x=x, y=y)
+                    self.sigRangeChangedManually.emit(self.state['mouseEnabled'])
 
 class ImageDraw(pg.ImageItem):
     """
@@ -232,7 +234,8 @@ class ImageDraw(pg.ImageItem):
 
     def mouseClickEvent(self, ev):
         if self.parent.masksOn or self.parent.outlinesOn:
-            if ev.button()==QtCore.Qt.RightButton and self.parent.loaded:
+            if  self.parent.loaded and (ev.button()==QtCore.Qt.RightButton or 
+                    ev.modifiers() == QtCore.Qt.ShiftModifier):
                 if not self.parent.in_stroke:
                     ev.accept()
                     self.create_start(ev.pos())
