@@ -1,23 +1,50 @@
+import os
 import numpy as np
 from matplotlib.colors import hsv_to_rgb, rgb_to_hsv
-from . import utils
 import cv2
 from scipy.ndimage import gaussian_filter
 import scipy
-import skimage
+import skimage.io
 from skimage import draw
+from skimage.segmentation import find_boundaries
 
-def show_segmentation(fig, img, maski, flowi, save_path=None):
-    #outpix = plot_outlines(maski)
-    outlines = masks_to_outlines(maski)
-    overlay = mask_overlay(img, maski)
+from . import utils
 
+
+def show_segmentation(fig, img, maski, flowi, channels=[0,0], file_name=None):
+    """ plot segmentation results (like on website)
+    
+    Can save each panel of figure with file_name option. Use channels option if
+    img input is not an RGB image with 3 channels.
+    
+    Parameters
+    -------------
+
+    fig: matplotlib.pyplot.figure
+        figure in which to make plot
+
+    img: 2D or 3D array
+        image input into cellpose
+
+    maski: int, 2D array
+        for image k, masks[k] output from Cellpose.eval, where 0=NO masks; 1,2,...=mask labels
+
+    flowi: int, 2D array 
+        for image k, flows[k][0] output from Cellpose.eval (RGB of flows)
+
+    channels: list of int (optional, default [0,0])
+        channels used to run Cellpose, no need to use if image is RGB
+
+    file_name: str (optional, default None)
+        file name of image, if file_name is not None, figure panels are saved
+
+    """
     ax = fig.add_subplot(1,4,1)
     img0 = img.copy()
     if img0.shape[0] < 4:
         img0 = np.transpose(img0, (1,2,0))
     if img0.shape[-1] < 3:
-        img0 = rgb_image(img0)
+        img0 = image_to_rgb(img0, channels=channels)
     else:
         if img0.max()<=50.0:
             img0 = np.uint8(np.clip(img0*255, 0, 1))
@@ -25,6 +52,9 @@ def show_segmentation(fig, img, maski, flowi, save_path=None):
     ax.set_title('original image')
     ax.axis('off')
 
+    outlines = masks_to_outlines(maski)
+    overlay = mask_overlay(img0, maski)
+
     ax = fig.add_subplot(1,4,2)
     outX, outY = np.nonzero(outlines)
     imgout= img0.copy()
@@ -45,43 +75,44 @@ def show_segmentation(fig, img, maski, flowi, save_path=None):
     ax.set_title('predicted cell pose')
     ax.axis('off')
 
-    if save_path is not None:
+    if file_name is not None:
+        save_path = os.path.splitext(file_name)[0]
         skimage.io.imsave(save_path + '_overlay.jpg', overlay)
         skimage.io.imsave(save_path + '_outlines.jpg', imgout)
         skimage.io.imsave(save_path + '_flows.jpg', flowi)
 
-
-
-def outline_overlay(img0, outlines, channels=[0,0], color=[255,0,0]):
-    """ outlines are red by default """
-    img = img0.copy()
-    if not isinstance(outlines, list):
-        outlines = [outlines]
-        color = [color]
-    for i in range(img.shape[-1]):
-        if np.ptp(img[:,:,i])>0:
-            img[:,:,i] = utils.normalize99(img[:,:,i])
-            img[:,:,i] = np.clip(img[:,:,i], 0, 1)
-    img *= 255
-    img = np.uint8(img)
-    RGB = np.zeros((img.shape[0], img.shape[1], 3), np.uint8)
-    if channels[0]==0:
-        RGB = np.tile(img[:,:,0][:,:,np.newaxis],(1,1,3))
-    else:
-        RGB[:,:,channels[0]-1] = img[:,:,0]
-        if channels[1] > 0:
-            RGB[:,:,channels[1]-1] = img[:,:,1]
-    for i in range(len(outlines)):
-        RGB[outlines[i]>0] = np.array(color[i])
-    return RGB
-
 def mask_overlay(img, masks, colors=None):
+    """ overlay masks on image (set image to grayscale)
+
+    Parameters
+    ----------------
+
+    img: int or float, 2D or 3D array
+        img is of size [Ly x Lx (x nchan)]
+
+    masks: int, 2D array
+        masks where 0=NO masks; 1,2,...=mask labels
+
+    colors: int, 2D array (optional, default None)
+        size [nmasks x 3], each entry is a color in 0-255 range
+
+    Returns
+    ----------------
+
+    RGB: uint8, 3D array
+        array of masks overlaid on grayscale image
+
+    """
     if colors is not None:
         if colors.max()>1:
             colors = np.float32(colors)
             colors /= 255
         colors = rgb_to_hsv(colors)
-    img = utils.normalize99(img.astype(np.float32).mean(axis=-1))
+    if img.ndim>2:
+        img = img.astype(np.float32).mean(axis=-1)
+    else:
+        img = img.astype(np.float32)
+    img = utils.normalize99(img)
     img -= img.min()
     img /= img.max()
     HSV = np.zeros((img.shape[0], img.shape[1], 3), np.float32)
@@ -96,25 +127,26 @@ def mask_overlay(img, masks, colors=None):
     RGB = (hsv_to_rgb(HSV) * 255).astype(np.uint8)
     return RGB
 
-def rgb_image(img):
+
+def image_to_rgb(img0, channels=[0,0]):
     """ image is 2 x Ly x Lx or Ly x Lx x 2 - change to RGB Ly x Lx x 3 """
-    if img.shape[-1]<3:
-        img = np.transpose(img, (2,0,1))
-    gray = False
-    if np.ptp(img[1]) < 1e-3:
-        gray = True
-    img[0] = utils.normalize99(img[0])
-    if not gray:
-        img[1] = utils.normalize99(img[1])
-    _,Ly,Lx = img.shape
-    img = np.clip(img, 0., 1.)
-    img = np.uint8(img*255)
-    RGB = np.zeros((Ly, Lx, 3), np.uint8)
-    if gray:
-        RGB = np.tile(img[0][...,np.newaxis], (1,1,3))
+    img = img0.copy()
+    img = img.astype(np.float32)
+    if img.shape[0]<3:
+        img = np.transpose(img, (1,2,0))
+    for i in range(img.shape[-1]):
+        if np.ptp(img[:,:,i])>0:
+            img[:,:,i] = utils.normalize99(img[:,:,i])
+            img[:,:,i] = np.clip(img[:,:,i], 0, 1)
+    img *= 255
+    img = np.uint8(img)
+    RGB = np.zeros((img.shape[0], img.shape[1], 3), np.uint8)
+    if channels[0]==0:
+        RGB = np.tile(img[:,:,0][:,:,np.newaxis],(1,1,3))
     else:
-        RGB[:,:,1] = img[0]
-        RGB[:,:,2] = img[1]
+        RGB[:,:,channels[0]-1] = img[:,:,0]
+        if channels[1] > 0:
+            RGB[:,:,channels[1]-1] = img[:,:,1]
     return RGB
 
 def interesting_patch(mask, bsize=130):
@@ -138,22 +170,12 @@ def disk(med, r, Ly, Lx):
     x = xx[inds].flatten()
     return y,x
 
-def circle(med, r):
-    """ returns pixels of circle with radius r and center med """
-    theta = np.linspace(0.0,2*np.pi,100)
-    x = r * np.cos(theta) + med[0]
-    y = r * np.sin(theta) + med[1]
-    x = x.astype(np.int32)
-    y = y.astype(np.int32)
-    return x,y
-
 def dx_to_circ(dP):
+    """ dP is 2 x Y x X => 'optic' flow representation """
     sc = max(np.percentile(dP[0], 99), np.percentile(dP[0], 1))
     Y = np.clip(dP[0] / sc, -1, 1)
     sc = max(np.percentile(dP[1], 99), np.percentile(dP[1], 1))
     X = np.clip(dP[1] / sc, -1, 1)
-    #Y = (np.clip(utils.normalize99(dP[0]), 0,1) - 0.5) * 2
-    #X = (np.clip(utils.normalize99(dP[1]), 0,1) - 0.5) * 2
     H = (np.arctan2(Y, X) + np.pi) / (2*np.pi)
     S = utils.normalize99(dP[0]**2 + dP[1]**2)
     V = np.ones_like(S)
@@ -163,72 +185,32 @@ def dx_to_circ(dP):
     return flow
 
 def masks_to_outlines(masks):
-    Ly, Lx = masks.shape
-    nmask = masks.max()
-    outlines = np.zeros((Ly,Lx), np.bool)
-    # pad T0 and mask by 2
-    T = np.zeros((Ly+4)*(Lx+4), np.int32)
-    Lx += 4
-    iun = np.unique(masks)[1:]
-    for iu in iun:
-        y,x = np.nonzero(masks==iu)
-        y+=1
-        x+=1
-        T[y*Lx + x] = 1
-        T[y*Lx + x] =  (T[(y-1)*Lx + x]   + T[(y+1)*Lx + x] +
-                        T[y*Lx + x-1]     + T[y*Lx + x+1] )
-        outlines[y-1,x-1] = np.logical_and(T[y*Lx+x]>0 , T[y*Lx+x]<4)
-        T[y*Lx + x] = 0
-    #outlines *= masks
+    """ get outlines of masks as a 0-1 array 
+    
+    Parameters
+    ----------------
+
+    masks: int, 2D array 
+        size [Ly x Lx], 0=NO masks; 1,2,...=mask labels
+
+    Returns
+    ----------------
+
+    outlines: bool, 2D array
+        size [Ly x Lx], True pixels are outlines
+
+    """
+    outlines = np.zeros(masks.shape, np.bool)
+    outlines[find_boundaries(masks, mode='inner')] = 1
     return outlines
 
-def masks_to_outlines_numbered(masks):
-    Ly, Lx = masks.shape
-    nmask = masks.max()
-    outlines = np.zeros((Ly,Lx), np.int32)
-    # pad T0 and mask by 2
-    T = np.zeros((Ly+2)*(Lx+2), np.int32)
-    Lx += 2
-    for k in range(nmask):
-        y,x = np.nonzero(masks==(k+1))
-        y+=1
-        x+=1
-        T[y*Lx + x] = 1
-        T[y*Lx + x] =  (T[(y-1)*Lx + x]   + T[(y+1)*Lx + x] +
-                        T[y*Lx + x-1]     + T[y*Lx + x+1] )
-        outlines[y-1,x-1] = (k+1)*np.logical_and(T[y*Lx+x]>0 , T[y*Lx+x]<4)
-        T[y*Lx + x] = 0
-    #outlines *= masks
-    return outlines
-
-def outline_from_mask(mask):
-    Y = mask.copy()
-    Ly, Lx = Y.shape
-    mu = np.zeros((Ly,Lx))
-    unq = np.unique(Y)
-    nmask = len(unq)-1
-    for j in range(nmask):
-        mask = (Y==unq[j+1])
-        y,x = (Y==unq[j+1]).nonzero()
-        y0 = np.min(y)
-        x0 = np.min(x)
-        y = y-y0
-        x = x-x0
-        Ly, Lx = np.max(y)+1, np.max(x)+1
-        ma0 = np.zeros((Ly,Lx))
-        ma0[y,x] = 1
-        ma = cv2.boxFilter(ma0, ksize=(3, 3), normalize=False, ddepth=-1)
-        maskE = np.logical_and(ma < 9, ma0>.5)
-        mu[y+y0,x+x0] = mu[y+y0,x+x0] +maskE[y,x]
-    return mu
-
-
-def plot_outlines(masks):
+def outlines_list(masks):
+    """ get outlines of masks as a list to loop over for plotting """
     outpix=[]
     for n in np.unique(masks)[1:]:
         mn = masks==n
         if mn.sum() > 0:
-            contours, hierarchy = cv2.findContours(mn.astype(np.uint8), mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_SIMPLE)
+            contours, _ = cv2.findContours(mn.astype(np.uint8), mode=cv2.RETR_EXTERNAL, method=cv2.CHAIN_APPROX_SIMPLE)
             #contours = measure.find_contours(mn, 0.5)
             cmax = np.argmax([c.shape[0] for c in contours])
             pix = contours[cmax].astype(int).squeeze()
